@@ -26,32 +26,34 @@ static SEXP pomp_default_rprocess (SEXP xstart, int nvars, int nreps, int ntimes
 
 SEXP do_rprocess (SEXP object, SEXP xstart, SEXP tstart, SEXP times, SEXP params, SEXP gnsi)
 {
-  int nprotect = 0;
+
   int *xdim, type, nvars, npars, nreps, nrepsx, ntimes;
   SEXP X, copy, rproc, args, accumvars, covar;
   SEXP dimXstart, dimP;
   const char *dimnm[3] = {"variable","rep","time"};
 
-  PROTECT(gnsi = duplicate(gnsi)); nprotect++;
+  PROTECT(gnsi = duplicate(gnsi));
 
-  PROTECT(tstart = AS_NUMERIC(tstart)); nprotect++;
+  PROTECT(tstart = AS_NUMERIC(tstart));
 
-  PROTECT(times = AS_NUMERIC(times)); nprotect++;
+  PROTECT(times = AS_NUMERIC(times));
   ntimes = length(times);
   if (ntimes < 1) {
     errorcall(R_NilValue,"length(times) < 1: no work to do.");
   }
 
-  PROTECT(xstart = as_matrix(xstart)); nprotect++;
-  PROTECT(dimXstart = GET_DIM(xstart)); nprotect++;
+  PROTECT(xstart = as_matrix(xstart));
+  PROTECT(dimXstart = GET_DIM(xstart));
   xdim = INTEGER(dimXstart);
   nvars = xdim[0]; nrepsx = xdim[1];
 
-  PROTECT(params = as_matrix(params)); nprotect++;
-  PROTECT(dimP = GET_DIM(params)); nprotect++;
+  PROTECT(params = as_matrix(params));
+  PROTECT(dimP = GET_DIM(params));
   xdim = INTEGER(dimP);
   npars = xdim[0]; nreps = xdim[1];
 
+  int nprotect = 7;
+
   if (nrepsx > nreps) {		// more ICs than parameters
     if (nrepsx % nreps != 0) {
       errorcall(R_NilValue,"the larger number of replicates is not a multiple of smaller.");
@@ -60,8 +62,9 @@ SEXP do_rprocess (SEXP object, SEXP xstart, SEXP tstart, SEXP times, SEXP params
       int dims[2];
       int j, k;
       dims[0] = npars; dims[1] = nrepsx;
-      PROTECT(copy = duplicate(params)); nprotect++;
-      PROTECT(params = makearray(2,dims)); nprotect++;
+      PROTECT(copy = duplicate(params));
+      PROTECT(params = makearray(2,dims));
+      nprotect += 2;
       setrownames(params,GET_ROWNAMES(GET_DIMNAMES(copy)),2);
       src = REAL(copy);
       tgt = REAL(params);
@@ -80,8 +83,9 @@ SEXP do_rprocess (SEXP object, SEXP xstart, SEXP tstart, SEXP times, SEXP params
       int dims[2];
       int j, k;
       dims[0] = nvars; dims[1] = nreps;
-      PROTECT(copy = duplicate(xstart)); nprotect++;
-      PROTECT(xstart = makearray(2,dims)); nprotect++;
+      PROTECT(copy = duplicate(xstart));
+      PROTECT(xstart = makearray(2,dims));
+      nprotect += 2;
       setrownames(xstart,GET_ROWNAMES(GET_DIMNAMES(copy)),2);
       src = REAL(copy);
       tgt = REAL(xstart);
@@ -93,10 +97,12 @@ SEXP do_rprocess (SEXP object, SEXP xstart, SEXP tstart, SEXP times, SEXP params
     }
   }
 
-  PROTECT(rproc = GET_SLOT(object,install("rprocess"))); nprotect++;
-  PROTECT(args = VectorToPairList(GET_SLOT(object,install("userdata")))); nprotect++;
-  PROTECT(accumvars = GET_SLOT(object,install("accumvars"))); nprotect++;
-  PROTECT(covar = GET_SLOT(object,install("covar"))); nprotect++;
+  PROTECT(rproc = GET_SLOT(object,install("rprocess")));
+  PROTECT(args = VectorToPairList(GET_SLOT(object,install("userdata"))));
+  PROTECT(accumvars = GET_SLOT(object,install("accumvars")));
+  PROTECT(covar = GET_SLOT(object,install("covar")));
+
+  nprotect += 4;
 
   // extract the process function
   type = *(INTEGER(GET_SLOT(rproc,install("type"))));
@@ -105,33 +111,37 @@ SEXP do_rprocess (SEXP object, SEXP xstart, SEXP tstart, SEXP times, SEXP params
   {
     SEXP fn;
     double deltat = 1.0;
-    PROTECT(fn = GET_SLOT(rproc,install("step.fn"))); nprotect++;
+    PROTECT(fn = GET_SLOT(rproc,install("step.fn")));
     PROTECT(X = euler_model_simulator(fn,xstart,tstart,times,params,deltat,type,
-      accumvars,covar,args,gnsi)); nprotect++;
+      accumvars,covar,args,gnsi));
+    nprotect += 2;
   }
     break;
   case discrete: case euler: // discrete-time and Euler
   {
     SEXP fn;
     double deltat;
-    PROTECT(fn = GET_SLOT(rproc,install("step.fn"))); nprotect++;
+    PROTECT(fn = GET_SLOT(rproc,install("step.fn")));
     deltat = *(REAL(AS_NUMERIC(GET_SLOT(rproc,install("delta.t")))));
     PROTECT(X = euler_model_simulator(fn,xstart,tstart,times,params,deltat,type,
-      accumvars,covar,args,gnsi)); nprotect++;
+      accumvars,covar,args,gnsi));
+    nprotect += 2;
   }
     break;
   case gill: // Gillespie's method
   {
     SEXP fn, vmatrix, hmax;
-    PROTECT(fn = GET_SLOT(rproc,install("rate.fn"))); nprotect++;
-    PROTECT(vmatrix = GET_SLOT(rproc,install("v"))); nprotect++;
-    PROTECT(hmax = GET_SLOT(rproc,install("hmax"))); nprotect++;
+    PROTECT(fn = GET_SLOT(rproc,install("rate.fn")));
+    PROTECT(vmatrix = GET_SLOT(rproc,install("v")));
+    PROTECT(hmax = GET_SLOT(rproc,install("hmax")));
     PROTECT(X = SSA_simulator(fn,xstart,tstart,times,params,vmatrix,covar,
-      accumvars,hmax,args,gnsi)); nprotect++;
+      accumvars,hmax,args,gnsi));
+    nprotect += 4;
   }
     break;
   case dflt: default:
-    PROTECT(X = pomp_default_rprocess(xstart,nvars,nreps,ntimes)); nprotect++;
+    PROTECT(X = pomp_default_rprocess(xstart,nvars,nreps,ntimes));
+    nprotect++;
     break;
   }
 
